@@ -1,4 +1,4 @@
-from scipy.stats import norm
+from winning.normaldist import normcdf, normpdf
 import numpy as np
 import math
 
@@ -97,7 +97,7 @@ def _unnormalized_skew_cdf(x, loc=0, scale=1.0, a=2.0):
     :return:  np.array length 2*L+1
     """
     t = (x - loc) / scale
-    return 2 / scale * norm.pdf(t) * norm.cdf(a * t)
+    return 2 / scale * normpdf(t) * normcdf(a * t)
 
 
 def sample_from_cdf(cdf, n_samples, unit=1.0, add_noise=False):
@@ -156,12 +156,9 @@ def sample_winner_of_many(densities, nSamples=5000):
     return density
 
 
-def expected_payoff(density, densityAll, multiplicityAll, cdf=None, cdfAll=None):
+def get_the_rest(density, densityAll, multiplicityAll, cdf=None, cdfAll=None):
     """ Returns expected _conditional_payoff_against_rest broken down by score,
        where _conditional_payoff_against_rest is 1 if we are better than rest (lower) and 1/(1+multiplicity) if we are equal
-
-
-
     """
     # Use np.sum( expected_payoff ) for the expectation
     if cdf is None:
@@ -197,8 +194,13 @@ def expected_payoff(density, densityAll, multiplicityAll, cdf=None, cdfAll=None)
     k = list(f1 == max(f1)).index(True)
     multiplicityRest[k:] = multiplicityRightTail[k:]
 
-    return _conditional_payoff_against_rest(density=density, densityRest=None, multiplicityRest=multiplicityRest,
-                                            cdf=cdf, cdfRest=cdfRest)
+    return cdfRest, multiplicityRest
+
+def expected_payoff(density, densityAll, multiplicityAll, cdf=None, cdfAll=None):
+    cdfRest, multiplicityRest = get_the_rest(density=density, densityAll=densityAll, multiplicityAll=multiplicityAll, cdf=cdf, cdfAll=cdfAll)
+    return _conditional_payoff_against_rest(density=density, densityRest=None, multiplicityRest=multiplicityRest, cdf=cdf, cdfRest=cdfRest)
+
+
 
 
 def _winner_of_two_pdf(densityA, densityB, multiplicityA=None, multiplicityB=None, cdfA=None, cdfB=None):
@@ -225,6 +227,22 @@ def _winner_of_two_pdf(densityA, densityB, multiplicityA=None, multiplicityB=Non
     return density, multiplicity
 
 
+def _loser_of_two_pdf(densityA, densityB):
+    reverse_density, reverse_multiplicity = _winner_of_two_pdf(np.flip(densityA), np.flip(densityB))
+    return np.flip(reverse_density), np.flip(reverse_multiplicity)
+
+
+def beats(densityA, multiplicityA, densityB, multiplicityB):
+    """
+        Returns expected _conditional_payoff_against_rest broken down by score
+    """
+    # use np.sum( _conditional_payoff_against_rest) for the expectation
+    cdfA = pdf_to_cdf(densityA)
+    cdfB = pdf_to_cdf(densityB)
+    win, draw, loss = _conditional_win_draw_loss(densityA, densityB, cdfA, cdfB)
+    return sum( win + draw * (1+multiplicityA) / (2 + multiplicityB + multiplicityA ) )
+
+
 def _conditional_win_draw_loss(densityA, densityB, cdfA, cdfB):
     """ Conditional win, draw and loss probability lattices for a two ratings race """
     win = densityA * (1 - cdfB)
